local uabStatus_MainBuffs = uabUnitStatus:NewModule("Main Buffs")

uabStatus_MainBuffs.aura = AceLibrary("SpecialEvents-Aura-2.0")
uabStatus_MainBuffs.defaultDB = {
    disabled = nil,
    intellect = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = nil,
            shaman = true,
            warlock = true,
            warrior = nil,
        },
	},
    fort = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    shadowprot = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    spirit = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = nil,
            shaman = true,
            warlock = true,
            warrior = nil,
        },
	},
    mark = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
}

uabStatus_MainBuffs.constants = {
    strings = {
        bintellect = "Buff: Arcane Intellect",
        bfort = "Buff: Fortitude",
        bshadowprot = "Buff: Shadow Protection",
        bspirit = "Buff: Divine Spirit",
        bmark = "Buff: Mark of the Wild",
    
        intellect1 = "Arcane Intellect",
        intellect2 = "Arcane Brilliance",
        fort1 = "Power Word: Fortitude",
        fort2 = "Prayer of Fortitude",
        shadowprot = "Shadow Protection",
        spirit1 = "Divine Spirit",
        spirit2 = "Prayer of Spirit",
        mark1 = "Mark of the Wild",
        mark2 = "Gift of the Wild",
    }
}

function uabStatus_MainBuffs:OnInitialize()
    self.super.OnInitialize(self)

	self:RegisterUnitStatus(uabStatus_MainBuffs.constants.strings.bintellect)
	self:RegisterUnitStatus(uabStatus_MainBuffs.constants.strings.bfort)
	self:RegisterUnitStatus(uabStatus_MainBuffs.constants.strings.bshadowprot)
	self:RegisterUnitStatus(uabStatus_MainBuffs.constants.strings.bspirit)
	self:RegisterUnitStatus(uabStatus_MainBuffs.constants.strings.bmark)
end

function uabStatus_MainBuffs:OnEnable()
end

function uabStatus_MainBuffs:OnDisable()
end

function uabStatus_MainBuffs:IsStatusGained(name, unit, action)
    local class = string.lower(UnitClass(unit))
    local hasbuff
    
	if name == uabStatus_MainBuffs.constants.strings.bintellect then
        if self.db.profile.intellect.classfilter[class] then
		    hasbuff = self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.intellect1) or self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.intellect2)
        end
	elseif name == uabStatus_MainBuffs.constants.strings.bfort then
        if self.db.profile.fort.classfilter[class] then
	        hasbuff = self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.fort1) or self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.fort2)
        end
	elseif name == uabStatus_MainBuffs.constants.strings.bshadowprot then
        if self.db.profile.shadowprot.classfilter[class] then
    	    hasbuff = self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.shadowprot)
        end
	elseif name == uabStatus_MainBuffs.constants.strings.bspirit then
        if self.db.profile.spirit.classfilter[class] then
	        hasbuff = self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.spirit1) or self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.spirit2)
        end
	elseif name == uabStatus_MainBuffs.constants.strings.bmark then
        if self.db.profile.mark.classfilter[class] then
    	    hasbuff = self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.mark1) or self.aura:UnitHasBuff(unit, uabStatus_MainBuffs.constants.strings.mark2)
        end
	end
    
    return hasbuff
end

function uabStatus_MainBuffs:OptionsForUnitStatus(name)
	if name == uabStatus_MainBuffs.constants.strings.bintellect then
		return self.db.profile.intellect
	elseif name == uabStatus_MainBuffs.constants.strings.bfort then
		return self.db.profile.fort
	elseif name == uabStatus_MainBuffs.constants.strings.bshadowprot then
		return self.db.profile.shadowprot
	elseif name == uabStatus_MainBuffs.constants.strings.bspirit then
		return self.db.profile.spirit
	elseif name == uabStatus_MainBuffs.constants.strings.bmark then
		return self.db.profile.mark
	end
end

function uabStatus_MainBuffs:CreateUnitStatusOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Buffs",
        desc = "Configure the options for Buffs unit status values",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of the Buffs unit status values",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                end,
            },

            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },

        },
    }

    args.args.intellect = self:CreateUnitStatusOption(self.db.profile.intellect, uabStatus_MainBuffs.constants.strings.bintellect, uabStatus_MainBuffs.constants.strings.bintellect)
    args.args.intellect.order = 10
    args.args.intellect.args.classfilter = self:CreateClassFilter(self.db.profile.intellect)
    args.args.intellect.args.classfilter.order = 40

    args.args.fort = self:CreateUnitStatusOption(self.db.profile.fort, uabStatus_MainBuffs.constants.strings.bfort, uabStatus_MainBuffs.constants.strings.bfort)
    args.args.fort.order = 20
    args.args.fort.args.classfilter = self:CreateClassFilter(self.db.profile.fort)
    args.args.fort.args.classfilter.order = 40

    args.args.spirit = self:CreateUnitStatusOption(self.db.profile.spirit, uabStatus_MainBuffs.constants.strings.bspirit, uabStatus_MainBuffs.constants.strings.bspirit)
    args.args.spirit.order = 30
    args.args.spirit.args.classfilter = self:CreateClassFilter(self.db.profile.spirit)
    args.args.spirit.args.classfilter.order = 40

    args.args.shadowprot = self:CreateUnitStatusOption(self.db.profile.shadowprot, uabStatus_MainBuffs.constants.strings.bshadowprot, uabStatus_MainBuffs.constants.strings.bshadowprot)
    args.args.shadowprot.order = 40
    args.args.shadowprot.args.classfilter = self:CreateClassFilter(self.db.profile.shadowprot)
    args.args.shadowprot.args.classfilter.order = 40

    args.args.mark = self:CreateUnitStatusOption(self.db.profile.mark, uabStatus_MainBuffs.constants.strings.bmark, uabStatus_MainBuffs.constants.strings.bmark)
    args.args.mark.order = 50
    args.args.mark.args.classfilter = self:CreateClassFilter(self.db.profile.mark)
    args.args.mark.args.classfilter.order = 40

    return args
end

function uabStatus_MainBuffs:CreateClassFilter(profile)
    if not profile.classfilter then
        profile.classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        }
    end
    
    local options = {
        type = 'group',
        name = "Class Filter",
        desc = "Set the class filter for this status",
        args = {
            druid = {
                type = 'toggle',
                name = "Druid",
                desc = "Check the status for a druid",
                order = 10,
                get = function() return profile.classfilter.druid end,
                set = function(v) profile.classfilter.druid = v end,
            },
            hunter = {
                type = 'toggle',
                name = "Hunter",
                desc = "Check the status for a hunter",
                order = 20,
                get = function() return profile.classfilter.hunter end,
                set = function(v) profile.classfilter.hunter = v end,
            },
            mage = {
                type = 'toggle',
                name = "Mage",
                desc = "Check the status for a mage",
                order = 30,
                get = function() return profile.classfilter.mage end,
                set = function(v) profile.classfilter.mage = v end,
            },
            paladin = {
                type = 'toggle',
                name = "Paladin",
                desc = "Check the status for a paladin",
                order = 40,
                get = function() return profile.classfilter.paladin end,
                set = function(v) profile.classfilter.paladin = v end,
            },
            priest = {
                type = 'toggle',
                name = "Priest",
                desc = "Check the status for a priest",
                order = 50,
                get = function() return profile.classfilter.priest end,
                set = function(v) profile.classfilter.priest = v end,
            },
            rogue = {
                type = 'toggle',
                name = "Rogue",
                desc = "Check the status for a rogue",
                order = 60,
                get = function() return profile.classfilter.rogue end,
                set = function(v) profile.classfilter.rogue = v end,
            },
            shaman = {
                type = 'toggle',
                name = "Shaman",
                desc = "Check the status for a shaman",
                order = 70,
                get = function() return profile.classfilter.shaman end,
                set = function(v) profile.classfilter.shaman = v end,
            },
            warlock = {
                type = 'toggle',
                name = "Warlock",
                desc = "Check the status for a warlock",
                order = 80,
                get = function() return profile.classfilter.warlock end,
                set = function(v) profile.classfilter.warlock = v end,
            },
            warrior = {
                type = 'toggle',
                name = "Warrior",
                desc = "Check the status for a warrior",
                order = 90,
                get = function() return profile.classfilter.warrior end,
                set = function(v) profile.classfilter.warrior = v end,
            },
        },
    }
    
    return options
end

